/// <reference types="node" />
import EventEmitter from 'events';
import { Script } from './Script';
import type { ScriptManagerConfig } from './types';
/**
 * A manager to ease resolution, downloading and executing additional code from:
 * - arbitrary JavaScript scripts
 * - Webpack chunks
 * - Webpack bundles
 * - Webpack MF containers
 *
 * Once an instance of `ScriptManager` is created, it will be globally available
 * under `ScriptManager.shared` in main bundle, chunks and containers.
 *
 * When instance of `ScriptManager` is already created, attempt to create new instance
 * will throw an exception.
 *
 * This API is mainly useful, if you are working with any form of Code Splitting.
 *
 * `ScriptManager` is also an `EventEmitter` and emits the following events:
 * - `resolving` with `{ scriptId, caller }`
 * - `resolved` with `scriptId: string, caller?: string, locator: NormalizedScriptLocator, cache: boolean`
 * - `prefetching` with `scriptId: string, caller?: string, locator: NormalizedScriptLocator, cache: boolean`
 * - `loading` with `scriptId: string, caller?: string, locator: NormalizedScriptLocator, cache: boolean`
 * - `loaded` with `scriptId: string, caller?: string, locator: NormalizedScriptLocator, cache: boolean`
 * - `error` with `error: Error`
 *
 * Example of using this API with async Webpack chunk:
 * ```js
 * import * as React from 'react';
 * import { ScriptManager, Script } from '@callstack/repack/client';
 *
 * new ScriptManager({
 *   resolve: async (scriptId) => {
 *     if (__DEV__) {
 *       return {
 *         url: Script.getDevServerURL(scriptId);
 *         cache: false,
 *       };
 *     }
 *
 *     return {
 *       url: Script.getRemoteURL(`http://domain.exaple/apps/${scriptId}`),
 *     };
 *   },
 * });
 *
 * // ScriptManager.loadScript is called internally when running `import()`
 * const TeacherModule = React.lazy(() => import('./Teacher.js'));
 * const StudentModule = React.lazy(() => import('./Student.js'));
 *
 * export function App({ role }) {
 *   if (role === 'teacher') {
 *     return <TeacherModule />;
 *   }
 *
 *   return <StudentModule />
 * }
 * ```
 */
export declare class ScriptManager extends EventEmitter {
    private nativeScriptManager;
    static get shared(): ScriptManager;
    private cache;
    private cacheInitialized;
    private resolve?;
    private storage?;
    /**
     * Constructs instance of `ScriptManager`, configures it to be able to resolve
     * location of scripts and optionally, it also allows to set up caching to
     * avoid over-fetching.
     *
     * @param config Configuration options.
     */
    constructor(config: ScriptManagerConfig, nativeScriptManager?: any);
    __destroy(): void;
    private initCache;
    private saveCache;
    private handleError;
    /**
     * Resolves a {@link Script} instance with normalized locator data.
     *
     * Use `ScriptManager.on('resolving', ({ scriptId, caller }) => { })` to listen for when
     * the script resolution begins.
     *
     * Use `ScriptManager.on('resolved', (script) => { })` to listen for when
     * the script's locator data is resolved.
     *
     * @param scriptId Id of the script to resolve.
     * @param caller Name of the calling script - it can be for example: name of the bundle, chunk or container.
     */
    resolveScript(scriptId: string, caller?: string, webpackContext?: import("./types").WebpackContext): Promise<Script>;
    /**
     * Resolves given script's location, downloads and executes it.
     * The execution of the code is handled internally by threading in React Native.
     *
     * Use `ScriptManager.on('loading', (script) => { })` to listen for when
     * the script is about to be loaded.
     *
     * Use `ScriptManager.on('loaded', (script) => { })` to listen for when
     * the script is loaded.
     *
     * @param scriptId Id of the script to load.
     * @param caller Name of the calling script - it can be for example: name of the bundle, chunk or container.
     */
    loadScript(scriptId: string, caller?: string, webpackContext?: import("./types").WebpackContext): Promise<void>;
    /**
     * Resolves given script's location and downloads it without executing.
     * This function can be awaited to detect if the script was downloaded and for error handling.
     *
     * Use `ScriptManager.on('prefetching', (script) => { })` to listen for when
     * the script's prefetch beings.
     *
     * @param scriptId Id of the script to prefetch.
     * @param caller Name of the calling script - it can be for example: name of the bundle, chunk or container.
     */
    prefetchScript(scriptId: string, caller?: string, webpackContext?: import("./types").WebpackContext): Promise<void>;
    /**
     * Clears the cache (if configured in {@link ScriptManager.configure}) and removes downloaded
     * files for given scripts from the filesystem. This function can be awaited to detect if the
     * scripts were invalidated and for error handling.
     *
     * Use `ScriptManager.on('invalidated', (scriptIds) => { })` to listen for when
     * the invalidation completes.
     *
     * @param scriptIds Array of script ids to clear from cache and remove from filesystem.
     */
    invalidateScripts(scriptIds?: string[]): Promise<void>;
}
