# Class: ScriptManager

A manager to ease resolution, downloading and executing additional code from:
- arbitrary JavaScript scripts
- Webpack chunks
- Webpack bundles
- Webpack MF containers

Once an instance of `ScriptManager` is created, it will be globally available
under `ScriptManager.shared` in main bundle, chunks and containers.

When instance of `ScriptManager` is already created, attempt to create new instance
will throw an exception.

This API is mainly useful, if you are working with any form of Code Splitting.

`ScriptManager` is also an `EventEmitter` and emits the following events:
- `resolving` with `{ scriptId, caller }`
- `resolved` with `scriptId: string, caller?: string, locator: NormalizedScriptLocator, cache: boolean`
- `prefetching` with `scriptId: string, caller?: string, locator: NormalizedScriptLocator, cache: boolean`
- `loading` with `scriptId: string, caller?: string, locator: NormalizedScriptLocator, cache: boolean`
- `loaded` with `scriptId: string, caller?: string, locator: NormalizedScriptLocator, cache: boolean`
- `error` with `error: Error`

Example of using this API with async Webpack chunk:
```js
import * as React from 'react';
import { ScriptManager, Script } from '@callstack/repack/client';

new ScriptManager({
  resolve: async (scriptId) => {
    if (__DEV__) {
      return {
        url: Script.getDevServerURL(scriptId);
        cache: false,
      };
    }

    return {
      url: Script.getRemoteURL(`http://domain.exaple/apps/${scriptId}`),
    };
  },
});

// ScriptManager.loadScript is called internally when running `import()`
const TeacherModule = React.lazy(() => import('./Teacher.js'));
const StudentModule = React.lazy(() => import('./Student.js'));

export function App({ role }) {
  if (role === 'teacher') {
    return <TeacherModule />;
  }

  return <StudentModule />
}
```

## Hierarchy

- `EventEmitter`

  ↳ **`ScriptManager`**

## Table of contents

### Constructors

- [constructor](ScriptManager.md#constructor)

### Properties

- [captureRejectionSymbol](ScriptManager.md#capturerejectionsymbol)
- [captureRejections](ScriptManager.md#capturerejections)
- [defaultMaxListeners](ScriptManager.md#defaultmaxlisteners)
- [errorMonitor](ScriptManager.md#errormonitor)

### Accessors

- [shared](ScriptManager.md#shared)

### Methods

- [\_\_destroy](ScriptManager.md#__destroy)
- [addListener](ScriptManager.md#addlistener)
- [emit](ScriptManager.md#emit)
- [eventNames](ScriptManager.md#eventnames)
- [getMaxListeners](ScriptManager.md#getmaxlisteners)
- [invalidateScripts](ScriptManager.md#invalidatescripts)
- [listenerCount](ScriptManager.md#listenercount)
- [listeners](ScriptManager.md#listeners)
- [loadScript](ScriptManager.md#loadscript)
- [off](ScriptManager.md#off)
- [on](ScriptManager.md#on)
- [once](ScriptManager.md#once)
- [prefetchScript](ScriptManager.md#prefetchscript)
- [prependListener](ScriptManager.md#prependlistener)
- [prependOnceListener](ScriptManager.md#prependoncelistener)
- [rawListeners](ScriptManager.md#rawlisteners)
- [removeAllListeners](ScriptManager.md#removealllisteners)
- [removeListener](ScriptManager.md#removelistener)
- [resolveScript](ScriptManager.md#resolvescript)
- [setMaxListeners](ScriptManager.md#setmaxlisteners)
- [listenerCount](ScriptManager.md#listenercount-1)
- [on](ScriptManager.md#on-1)
- [once](ScriptManager.md#once-1)

## Constructors

### constructor

• **new ScriptManager**(`config`, `nativeScriptManager?`)

Constructs instance of `ScriptManager`, configures it to be able to resolve
location of scripts and optionally, it also allows to set up caching to
avoid over-fetching.

#### Parameters

| Name | Type | Default value | Description |
| :------ | :------ | :------ | :------ |
| `config` | [`ScriptManagerConfig`](../interfaces/ScriptManagerConfig.md) | `undefined` | Configuration options. |
| `nativeScriptManager` | `any` | `NativeModules.ScriptManager` | - |

#### Overrides

EventEmitter.constructor

#### Defined in

[packages/repack/src/modules/ScriptManager/ScriptManager.ts:101](https://github.com/callstack/repack/blob/a067171/packages/repack/src/modules/ScriptManager/ScriptManager.ts#L101)

## Properties

### captureRejectionSymbol

▪ `Static` `Readonly` **captureRejectionSymbol**: typeof [`captureRejectionSymbol`](ScriptManager.md#capturerejectionsymbol)

#### Inherited from

EventEmitter.captureRejectionSymbol

#### Defined in

node_modules/@types/node/events.d.ts:38

___

### captureRejections

▪ `Static` **captureRejections**: `boolean`

Sets or gets the default captureRejection value for all emitters.

#### Inherited from

EventEmitter.captureRejections

#### Defined in

node_modules/@types/node/events.d.ts:44

___

### defaultMaxListeners

▪ `Static` **defaultMaxListeners**: `number`

#### Inherited from

EventEmitter.defaultMaxListeners

#### Defined in

node_modules/@types/node/events.d.ts:45

___

### errorMonitor

▪ `Static` `Readonly` **errorMonitor**: typeof [`errorMonitor`](ScriptManager.md#errormonitor)

This symbol shall be used to install a listener for only monitoring `'error'`
events. Listeners installed using this symbol are called before the regular
`'error'` listeners are called.

Installing a listener using this symbol does not change the behavior once an
`'error'` event is emitted, therefore the process will still crash if no
regular `'error'` listener is installed.

#### Inherited from

EventEmitter.errorMonitor

#### Defined in

node_modules/@types/node/events.d.ts:37

## Accessors

### shared

• `Static` `get` **shared**(): [`ScriptManager`](ScriptManager.md)

#### Returns

[`ScriptManager`](ScriptManager.md)

#### Defined in

[packages/repack/src/modules/ScriptManager/ScriptManager.ts:79](https://github.com/callstack/repack/blob/a067171/packages/repack/src/modules/ScriptManager/ScriptManager.ts#L79)

## Methods

### \_\_destroy

▸ **__destroy**(): `void`

#### Returns

`void`

#### Defined in

[packages/repack/src/modules/ScriptManager/ScriptManager.ts:133](https://github.com/callstack/repack/blob/a067171/packages/repack/src/modules/ScriptManager/ScriptManager.ts#L133)

___

### addListener

▸ **addListener**(`event`, `listener`): [`ScriptManager`](ScriptManager.md)

#### Parameters

| Name | Type |
| :------ | :------ |
| `event` | `string` \| `symbol` |
| `listener` | (...`args`: `any`[]) => `void` |

#### Returns

[`ScriptManager`](ScriptManager.md)

#### Inherited from

EventEmitter.addListener

#### Defined in

node_modules/@types/node/events.d.ts:57

___

### emit

▸ **emit**(`event`, ...`args`): `boolean`

#### Parameters

| Name | Type |
| :------ | :------ |
| `event` | `string` \| `symbol` |
| `...args` | `any`[] |

#### Returns

`boolean`

#### Inherited from

EventEmitter.emit

#### Defined in

node_modules/@types/node/events.d.ts:67

___

### eventNames

▸ **eventNames**(): (`string` \| `symbol`)[]

#### Returns

(`string` \| `symbol`)[]

#### Inherited from

EventEmitter.eventNames

#### Defined in

node_modules/@types/node/events.d.ts:72

___

### getMaxListeners

▸ **getMaxListeners**(): `number`

#### Returns

`number`

#### Inherited from

EventEmitter.getMaxListeners

#### Defined in

node_modules/@types/node/events.d.ts:64

___

### invalidateScripts

▸ **invalidateScripts**(`scriptIds?`): `Promise`<`void`\>

Clears the cache (if configured in {@link ScriptManager.configure}) and removes downloaded
files for given scripts from the filesystem. This function can be awaited to detect if the
scripts were invalidated and for error handling.

Use `ScriptManager.on('invalidated', (scriptIds) => { })` to listen for when
the invalidation completes.

#### Parameters

| Name | Type | Default value | Description |
| :------ | :------ | :------ | :------ |
| `scriptIds` | `string`[] | `[]` | Array of script ids to clear from cache and remove from filesystem. |

#### Returns

`Promise`<`void`\>

#### Defined in

[packages/repack/src/modules/ScriptManager/ScriptManager.ts:308](https://github.com/callstack/repack/blob/a067171/packages/repack/src/modules/ScriptManager/ScriptManager.ts#L308)

___

### listenerCount

▸ **listenerCount**(`event`): `number`

#### Parameters

| Name | Type |
| :------ | :------ |
| `event` | `string` \| `symbol` |

#### Returns

`number`

#### Inherited from

EventEmitter.listenerCount

#### Defined in

node_modules/@types/node/events.d.ts:68

___

### listeners

▸ **listeners**(`event`): `Function`[]

#### Parameters

| Name | Type |
| :------ | :------ |
| `event` | `string` \| `symbol` |

#### Returns

`Function`[]

#### Inherited from

EventEmitter.listeners

#### Defined in

node_modules/@types/node/events.d.ts:65

___

### loadScript

▸ **loadScript**(`scriptId`, `caller?`, `webpackContext?`): `Promise`<`void`\>

Resolves given script's location, downloads and executes it.
The execution of the code is handled internally by threading in React Native.

Use `ScriptManager.on('loading', (script) => { })` to listen for when
the script is about to be loaded.

Use `ScriptManager.on('loaded', (script) => { })` to listen for when
the script is loaded.

#### Parameters

| Name | Type | Description |
| :------ | :------ | :------ |
| `scriptId` | `string` | Id of the script to load. |
| `caller?` | `string` | Name of the calling script - it can be for example: name of the bundle, chunk or container. |
| `webpackContext` | [`WebpackContext`](../interfaces/WebpackContext.md) | - |

#### Returns

`Promise`<`void`\>

#### Defined in

[packages/repack/src/modules/ScriptManager/ScriptManager.ts:231](https://github.com/callstack/repack/blob/a067171/packages/repack/src/modules/ScriptManager/ScriptManager.ts#L231)

___

### off

▸ **off**(`event`, `listener`): [`ScriptManager`](ScriptManager.md)

#### Parameters

| Name | Type |
| :------ | :------ |
| `event` | `string` \| `symbol` |
| `listener` | (...`args`: `any`[]) => `void` |

#### Returns

[`ScriptManager`](ScriptManager.md)

#### Inherited from

EventEmitter.off

#### Defined in

node_modules/@types/node/events.d.ts:61

___

### on

▸ **on**(`event`, `listener`): [`ScriptManager`](ScriptManager.md)

#### Parameters

| Name | Type |
| :------ | :------ |
| `event` | `string` \| `symbol` |
| `listener` | (...`args`: `any`[]) => `void` |

#### Returns

[`ScriptManager`](ScriptManager.md)

#### Inherited from

EventEmitter.on

#### Defined in

node_modules/@types/node/events.d.ts:58

___

### once

▸ **once**(`event`, `listener`): [`ScriptManager`](ScriptManager.md)

#### Parameters

| Name | Type |
| :------ | :------ |
| `event` | `string` \| `symbol` |
| `listener` | (...`args`: `any`[]) => `void` |

#### Returns

[`ScriptManager`](ScriptManager.md)

#### Inherited from

EventEmitter.once

#### Defined in

node_modules/@types/node/events.d.ts:59

___

### prefetchScript

▸ **prefetchScript**(`scriptId`, `caller?`, `webpackContext?`): `Promise`<`void`\>

Resolves given script's location and downloads it without executing.
This function can be awaited to detect if the script was downloaded and for error handling.

Use `ScriptManager.on('prefetching', (script) => { })` to listen for when
the script's prefetch beings.

#### Parameters

| Name | Type | Description |
| :------ | :------ | :------ |
| `scriptId` | `string` | Id of the script to prefetch. |
| `caller?` | `string` | Name of the calling script - it can be for example: name of the bundle, chunk or container. |
| `webpackContext` | [`WebpackContext`](../interfaces/WebpackContext.md) | - |

#### Returns

`Promise`<`void`\>

#### Defined in

[packages/repack/src/modules/ScriptManager/ScriptManager.ts:277](https://github.com/callstack/repack/blob/a067171/packages/repack/src/modules/ScriptManager/ScriptManager.ts#L277)

___

### prependListener

▸ **prependListener**(`event`, `listener`): [`ScriptManager`](ScriptManager.md)

#### Parameters

| Name | Type |
| :------ | :------ |
| `event` | `string` \| `symbol` |
| `listener` | (...`args`: `any`[]) => `void` |

#### Returns

[`ScriptManager`](ScriptManager.md)

#### Inherited from

EventEmitter.prependListener

#### Defined in

node_modules/@types/node/events.d.ts:70

___

### prependOnceListener

▸ **prependOnceListener**(`event`, `listener`): [`ScriptManager`](ScriptManager.md)

#### Parameters

| Name | Type |
| :------ | :------ |
| `event` | `string` \| `symbol` |
| `listener` | (...`args`: `any`[]) => `void` |

#### Returns

[`ScriptManager`](ScriptManager.md)

#### Inherited from

EventEmitter.prependOnceListener

#### Defined in

node_modules/@types/node/events.d.ts:71

___

### rawListeners

▸ **rawListeners**(`event`): `Function`[]

#### Parameters

| Name | Type |
| :------ | :------ |
| `event` | `string` \| `symbol` |

#### Returns

`Function`[]

#### Inherited from

EventEmitter.rawListeners

#### Defined in

node_modules/@types/node/events.d.ts:66

___

### removeAllListeners

▸ **removeAllListeners**(`event?`): [`ScriptManager`](ScriptManager.md)

#### Parameters

| Name | Type |
| :------ | :------ |
| `event?` | `string` \| `symbol` |

#### Returns

[`ScriptManager`](ScriptManager.md)

#### Inherited from

EventEmitter.removeAllListeners

#### Defined in

node_modules/@types/node/events.d.ts:62

___

### removeListener

▸ **removeListener**(`event`, `listener`): [`ScriptManager`](ScriptManager.md)

#### Parameters

| Name | Type |
| :------ | :------ |
| `event` | `string` \| `symbol` |
| `listener` | (...`args`: `any`[]) => `void` |

#### Returns

[`ScriptManager`](ScriptManager.md)

#### Inherited from

EventEmitter.removeListener

#### Defined in

node_modules/@types/node/events.d.ts:60

___

### resolveScript

▸ **resolveScript**(`scriptId`, `caller?`, `webpackContext?`): `Promise`<[`Script`](Script.md)\>

Resolves a [Script](Script.md) instance with normalized locator data.

Use `ScriptManager.on('resolving', ({ scriptId, caller }) => { })` to listen for when
the script resolution begins.

Use `ScriptManager.on('resolved', (script) => { })` to listen for when
the script's locator data is resolved.

#### Parameters

| Name | Type | Description |
| :------ | :------ | :------ |
| `scriptId` | `string` | Id of the script to resolve. |
| `caller?` | `string` | Name of the calling script - it can be for example: name of the bundle, chunk or container. |
| `webpackContext` | [`WebpackContext`](../interfaces/WebpackContext.md) | - |

#### Returns

`Promise`<[`Script`](Script.md)\>

#### Defined in

[packages/repack/src/modules/ScriptManager/ScriptManager.ts:173](https://github.com/callstack/repack/blob/a067171/packages/repack/src/modules/ScriptManager/ScriptManager.ts#L173)

___

### setMaxListeners

▸ **setMaxListeners**(`n`): [`ScriptManager`](ScriptManager.md)

#### Parameters

| Name | Type |
| :------ | :------ |
| `n` | `number` |

#### Returns

[`ScriptManager`](ScriptManager.md)

#### Inherited from

EventEmitter.setMaxListeners

#### Defined in

node_modules/@types/node/events.d.ts:63

___

### listenerCount

▸ `Static` **listenerCount**(`emitter`, `event`): `number`

**`deprecated`** since v4.0.0

#### Parameters

| Name | Type |
| :------ | :------ |
| `emitter` | `EventEmitter` |
| `event` | `string` \| `symbol` |

#### Returns

`number`

#### Inherited from

EventEmitter.listenerCount

#### Defined in

node_modules/@types/node/events.d.ts:26

___

### on

▸ `Static` **on**(`emitter`, `event`): `AsyncIterableIterator`<`any`\>

#### Parameters

| Name | Type |
| :------ | :------ |
| `emitter` | `EventEmitter` |
| `event` | `string` |

#### Returns

`AsyncIterableIterator`<`any`\>

#### Inherited from

EventEmitter.on

#### Defined in

node_modules/@types/node/events.d.ts:23

___

### once

▸ `Static` **once**(`emitter`, `event`): `Promise`<`any`[]\>

#### Parameters

| Name | Type |
| :------ | :------ |
| `emitter` | `NodeEventTarget` |
| `event` | `string` \| `symbol` |

#### Returns

`Promise`<`any`[]\>

#### Inherited from

EventEmitter.once

#### Defined in

node_modules/@types/node/events.d.ts:21

▸ `Static` **once**(`emitter`, `event`): `Promise`<`any`[]\>

#### Parameters

| Name | Type |
| :------ | :------ |
| `emitter` | `DOMEventTarget` |
| `event` | `string` |

#### Returns

`Promise`<`any`[]\>

#### Inherited from

EventEmitter.once

#### Defined in

node_modules/@types/node/events.d.ts:22
